(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('tslib'), require('apollo-link'), require('apollo-link-http-common'), require('apollo-link-batch')) :
  typeof define === 'function' && define.amd ? define(['exports', 'tslib', 'apollo-link', 'apollo-link-http-common', 'apollo-link-batch'], factory) :
  (global = global || self, factory((global.apolloLink = global.apolloLink || {}, global.apolloLink.batchHttp = {}), global.tslib, global.apolloLink.core, global.apolloLink.httpCommon, global.apolloLink.batch));
}(this, (function (exports, tslib_1, apolloLink, apolloLinkHttpCommon, apolloLinkBatch) { 'use strict';

  var BatchHttpLink = (function (_super) {
      tslib_1.__extends(BatchHttpLink, _super);
      function BatchHttpLink(fetchParams) {
          var _this = _super.call(this) || this;
          var _a = fetchParams || {}, _b = _a.uri, uri = _b === void 0 ? '/graphql' : _b, fetcher = _a.fetch, includeExtensions = _a.includeExtensions, batchInterval = _a.batchInterval, batchMax = _a.batchMax, batchKey = _a.batchKey, requestOptions = tslib_1.__rest(_a, ["uri", "fetch", "includeExtensions", "batchInterval", "batchMax", "batchKey"]);
          apolloLinkHttpCommon.checkFetcher(fetcher);
          if (!fetcher) {
              fetcher = fetch;
          }
          var linkConfig = {
              http: { includeExtensions: includeExtensions },
              options: requestOptions.fetchOptions,
              credentials: requestOptions.credentials,
              headers: requestOptions.headers,
          };
          _this.batchInterval = batchInterval || 10;
          _this.batchMax = batchMax || 10;
          var batchHandler = function (operations) {
              var chosenURI = apolloLinkHttpCommon.selectURI(operations[0], uri);
              var context = operations[0].getContext();
              var clientAwarenessHeaders = {};
              if (context.clientAwareness) {
                  var _a = context.clientAwareness, name_1 = _a.name, version = _a.version;
                  if (name_1) {
                      clientAwarenessHeaders['apollographql-client-name'] = name_1;
                  }
                  if (version) {
                      clientAwarenessHeaders['apollographql-client-version'] = version;
                  }
              }
              var contextConfig = {
                  http: context.http,
                  options: context.fetchOptions,
                  credentials: context.credentials,
                  headers: tslib_1.__assign({}, clientAwarenessHeaders, context.headers),
              };
              var optsAndBody = operations.map(function (operation) {
                  return apolloLinkHttpCommon.selectHttpOptionsAndBody(operation, apolloLinkHttpCommon.fallbackHttpConfig, linkConfig, contextConfig);
              });
              var loadedBody = optsAndBody.map(function (_a) {
                  var body = _a.body;
                  return body;
              });
              var options = optsAndBody[0].options;
              if (options.method === 'GET') {
                  return apolloLink.fromError(new Error('apollo-link-batch-http does not support GET requests'));
              }
              try {
                  options.body = apolloLinkHttpCommon.serializeFetchParameter(loadedBody, 'Payload');
              }
              catch (parseError) {
                  return apolloLink.fromError(parseError);
              }
              var controller;
              if (!options.signal) {
                  var _b = apolloLinkHttpCommon.createSignalIfSupported(), _controller = _b.controller, signal = _b.signal;
                  controller = _controller;
                  if (controller)
                      options.signal = signal;
              }
              return new apolloLink.Observable(function (observer) {
                  fetcher(chosenURI, options)
                      .then(function (response) {
                      operations.forEach(function (operation) { return operation.setContext({ response: response }); });
                      return response;
                  })
                      .then(apolloLinkHttpCommon.parseAndCheckHttpResponse(operations))
                      .then(function (result) {
                      observer.next(result);
                      observer.complete();
                      return result;
                  })
                      .catch(function (err) {
                      if (err.name === 'AbortError')
                          return;
                      if (err.result && err.result.errors && err.result.data) {
                          observer.next(err.result);
                      }
                      observer.error(err);
                  });
                  return function () {
                      if (controller)
                          controller.abort();
                  };
              });
          };
          batchKey =
              batchKey ||
                  (function (operation) {
                      var context = operation.getContext();
                      var contextConfig = {
                          http: context.http,
                          options: context.fetchOptions,
                          credentials: context.credentials,
                          headers: context.headers,
                      };
                      return apolloLinkHttpCommon.selectURI(operation, uri) + JSON.stringify(contextConfig);
                  });
          _this.batcher = new apolloLinkBatch.BatchLink({
              batchInterval: _this.batchInterval,
              batchMax: _this.batchMax,
              batchKey: batchKey,
              batchHandler: batchHandler,
          });
          return _this;
      }
      BatchHttpLink.prototype.request = function (operation) {
          return this.batcher.request(operation);
      };
      return BatchHttpLink;
  }(apolloLink.ApolloLink));

  exports.BatchHttpLink = BatchHttpLink;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=bundle.umd.js.map
